/****************************************************************************
 *
 * Julia set test program for PS3 cell architecture.
 *
 * (c) 2007 Ranulf Doswell <dev@ranulf.net>
 *
 * I've only been programming the PS3 for just over 24 hours and here is my
 * first attempt at just the SPUs, DMA transfers and hand-optimising SBU
 * assembler (such a nice processor to experiment with)...
 *
 * Please feel free to use this as a basis for your experimentations, but if
 * you come up with anything cool, please let me know!
 *
 ****************************************************************************/

#include <fcntl.h>
#include <stdint.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <linux/kd.h>
#include <sys/time.h>
#include <linux/fb.h>
#include <asm/ps3fb.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <unistd.h>

class FrameBuffer {
public:
	FrameBuffer();
	~FrameBuffer();

    void* getFrameBuffer();     // gets the current off-screen framebuffer
    void* getOnScreenFrameBuffer();  // gets the currently visible framebuffer
    void flip();                // flips to the next off-screen frame
    void vsync();               // wait for vsync
    float getFPS();               // get FPS rate to date
    int getFramecount();        // get framecount to date

    uint32_t width;             // width of the screen
    uint32_t height;            // height of the screen

    uint32_t left;              // offset into the viewpoint when not fullscreen
    uint32_t top;               // offset into the viewpoint when not fullscreen

    uint32_t right;             // end of the viewpoint when not fullscreen
    uint32_t bottom;            // end of the viewpoint when not fullscreen

private:
	void switchMode(bool gfxMode);
	int consoleFD;

    void* addr;
    void* addr2;
    void* current;
    int length;
    bool frame;

    struct timeval tv;
    int framecount;
};
