/****************************************************************************
 *
 * Julia set test program for PS3 cell architecture.
 *
 * (c) 2007 Ranulf Doswell <dev@ranulf.net>
 *
 * I've only been programming the PS3 for just over 24 hours and here is my
 * first attempt at just the SPUs, DMA transfers and hand-optimising SBU
 * assembler (such a nice processor to experiment with)...
 *
 * Please feel free to use this as a basis for your experimentations, but if
 * you come up with anything cool, please let me know!
 *
 ****************************************************************************/

#include "fb.h"
#include "spu.h"
#include "protocol.h"

#define DEFINE_ACTIVE_PROGRAM(x) extern spe_program_handle_t x; \
							     spe_program_handle_t* program = &x;

#ifndef UNOPT
DEFINE_ACTIVE_PROGRAM(optimised_handle)
#else
DEFINE_ACTIVE_PROGRAM(julia_spe_handle)
#endif

#ifdef TIMETEST
#define NUM_SPUS 6
#define FRAME_STEP 5
#define FRAME_START 1000
#define FRAME_END 9500
#else
#define NUM_SPUS 6
#define FRAME_STEP 1
#define FRAME_START 10
#define FRAME_END 10500
//#define VSYNC
#endif

inline void* aligned_malloc(int length) {
	void* p = malloc(length+15);
	unsigned long ptr = (unsigned long) p;
	void* p2 = (void*) ( (ptr+15) & ~15);
	//printf("Aligning %08lx -> %08lx\n", p, p2);
	//char** pp =(char**)p2;
	//p2[-1]=p;
	return p2;
}

inline void aligned_free(void *p) {
//	char** pp =(char**)p;
//	free(p[-1]);
}

int main(int argc, char *argv[]) {
    uint32_t frame = 0;
    struct timeval tv;
    uint32_t time;

	if (sizeof(program_data) & 15) {
		printf("FATAL error: program_data structure size is %d bytes long.\n",
			sizeof(program_data));
		exit(1);
	}


    FrameBuffer fb;

	Spu* spu[NUM_SPUS];
	program_data* data[NUM_SPUS];
		
	for (int s=0; s<NUM_SPUS; s++) {
		data[s]=(program_data*)aligned_malloc(sizeof(program_data));
#ifdef MESSAGES
		spu[s]=new Spu(program, data[s], NULL);
#else	
		spu[s]=NULL;
#endif
	}

	for (int s=FRAME_START; s<FRAME_END; s+=FRAME_STEP) {
//		printf("Frame %d\n",s);
        fb.flip();

		int block = 15;

		int sindex = 0;
		
		for (int y=0; y<fb.height;y+=block) {
#ifndef TIMETEST
			data[sindex]->corner_x = -0.25;
			data[sindex]->corner_y = -0.43;
			data[sindex]->x0 = -1.0 / 1.2 * (s+1)/4500.0;
			data[sindex]->y0 = -1.0 + (s+1)/7700.0;
			data[sindex]->stepx_xinc = 1.3 *(1.0+(4000-s)/8700.0) / fb.width;
			data[sindex]->stepy_yinc = data[sindex]->stepx_xinc;
			data[sindex]->stepx_yinc = 0.0;
			data[sindex]->stepy_xinc = 0.0;
#else
			data[sindex]->corner_x = -0.8;
			data[sindex]->corner_y = -0.6;
			data[sindex]->x0 = -1.0 / 1.2 * (s+1)/4000.0;
			data[sindex]->y0 = -1.0 + (s+1)/8400.0;
			data[sindex]->stepx_xinc = 1.7 / fb.width;
			data[sindex]->stepy_yinc = 1.7 / fb.height * 0.75;
			data[sindex]->stepx_yinc = 0.0;
			data[sindex]->stepy_xinc = 0.0;
#endif
			data[sindex]->framebuffer = fb.getFrameBuffer();
			data[sindex]->width = fb.width;
			data[sindex]->height = fb.height;

			data[sindex]->start = y;
			data[sindex]->count = (y+block>fb.height) ? (fb.height-y) : block;
#ifdef MESSAGES
			data[sindex]->tag = s;
			spu[sindex]->write(sindex);
			spu[sindex]->read();
#else	
			if (spu[sindex])
				spu[sindex]->join();
			spu[sindex]=new Spu(program, data[sindex], NULL);
#endif
			sindex = (sindex+1) % NUM_SPUS;
		}
#ifndef MESSAGES
		for (int q=0; q<NUM_SPUS; q++) {
			if (spu[q]) {
				spu[q]->join();
				spu[q]=NULL;
			}
		}
#endif
		for (int pause=0; pause<1; pause++) {
#ifdef VSYNC
           fb.vsync();
#endif
}
    }

	for (int q=0; q<NUM_SPUS; q++) {
		if (spu[q]) {
#ifdef MESSAGES
			spu[q]->write(-1);
#endif
			spu[q]->join();
			delete spu[q];
		}
		//aligned_free(data[q]);
	}

    printf("Average framecount over %d frames: %.1f FPS\n", fb.getFramecount(),
        fb.getFPS());
}
